import logging
import os

from .exceptions import SchemaValidationError
from .http_client import HTTPClient
from .schema import (ArticleSummarySchema,
                     SourceSchema, ValidationError)

log = logging.getLogger(__name__)

headers = {
    'Accept-Encoding':
        'gzip, deflate, sdch',
    'Accept-Language':
        'en-US,en;q=0.8',
    'Upgrade-Insecure-Requests':
        '1',
    'User-Agent':
        'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_12_4) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/59.0.3071.115 Safari/537.36',
}

DEFAULT_USER_AGENT = 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_12_4) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/59.0.3071.115 Safari/537.36'

VALIDATION_LEVELS = ('STRICT', 'DROP', 'ALLOW')


def parse_date(date_str):
    from dateutil.parser import parse
    parsed_date = parse(date_str)
    return parsed_date.isoformat()


class NewsAPI:

    def __init__(self, api_key: str = None, user_agent: str = DEFAULT_USER_AGENT, proxy: str = None,
                 validation: str = 'ALLOW'):
        """

        :param api_key:
        :param user_agent:
        :param proxy:
        :param validation:
        """
        if validation in VALIDATION_LEVELS:
            self.validation = validation
        else:
            raise ValueError('Validation level must be one of STRICT, DROP, or ALLOW')
        if api_key:
            self.api_key = api_key
        else:
            if os.environ.get('NEWS_API_KEY'):
                self.api_key = os.environ.get('NEWS_API_KEY')
            else:
                raise ValueError(
                    'News API key must be provided as an argument or as an environment variable NEWS_API_KEY')
        self.user_agent = user_agent
        self.http_client = HTTPClient(user_agent=user_agent, headers=headers, proxy=proxy)

    async def sources(self):
        """
        Retrieve sources from newsapi.org
        """
        url = 'https://newsapi.org/v1/sources?apiKey={}'.format(self.api_key)

        try:
            resp = await self.http_client.get(url=url, is_json=True)
        except Exception as error:
            raise

        sources = resp.get('sources', [])

        if self.validation is 'ALLOW':
            return SourceSchema(strict=False).load(
                sources, many=True).data

        try:
            return SourceSchema(strict=True).load(
                sources, many=True).data
        except ValidationError as error:
            if self.validation is 'DROP':
                loaded_sources = SourceSchema(strict=False).load(
                    sources, many=True).data
                for index in error.messages.keys():
                    del loaded_sources[index]
                return loaded_sources
            raise SchemaValidationError('Unable to validate response schema')

    async def latest_articles(self, source_name: str, sort: str):
        """
        Retrieve articles by source and sort
        :param source_name:
        :param sort:
        :return:
        """

        url = 'https://newsapi.org/v1/articles?source={}&sortBy={}&apiKey={}' \
            .format(source_name, sort, self.api_key)

        try:
            resp = await self.http_client.get(url=url, is_json=True)
        except Exception as error:
            raise

        articles = self._extract_features(resp)

        if self.validation is 'ALLOW':
            return ArticleSummarySchema(strict=False).load(
                articles, many=True).data

        try:
            return ArticleSummarySchema(strict=True).load(
                articles, many=True).data
        except ValidationError as error:
            if self.validation is 'DROP':
                loaded_articles = ArticleSummarySchema(strict=False).load(
                    articles, many=True).data
                for index in error.messages.keys():
                    del loaded_articles[index]
                return loaded_articles
            raise SchemaValidationError('Unable to validate response schema')

    def _extract_features(self, resp):
        parsed_articles = []
        source = resp.get('source')
        articles = resp.get('articles', None)
        for article in articles:
            published = article.get('publishedAt', None)
            try:
                published_dt = parse_date(published)
            except (TypeError, ValueError):
                published_dt = None
            article_dict = dict(
                id=article.get('url'),
                title=article.get('title', None),
                published_at=published_dt,
                author=article.get('author', None),
                summary=article.get('description'),
                url=article.get('url'),
            )
            parsed_articles.append(article_dict)
        return parsed_articles



from marshmallow import Schema, ValidationError, fields


class SchemaValidationError(ValidationError):
    pass


class SourceSchema(Schema):
    id = fields.Str(required=True)
    name = fields.Str(required=True)
    description = fields.Str(required=True)
    url = fields.Url(required=True)
    category = fields.Str(required=True)
    langugage = fields.Str(required=True)
    country = fields.Str(required=True)
    sorts_available = fields.List(fields.Str(), load_from='sortBysAvailable')


class ArticleSummarySchema(Schema):
    published_at = fields.DateTime(required=True)
    author = fields.Str(required=True)
    title = fields.Str(required=True)
    summary = fields.Str(required=True)
    url = fields.Url(required=True)


class ArticleSchema(Schema):
    url = fields.Url(required=True)
    body = fields.Str(required=True)
    title = fields.Str(required=True)
    authors = fields.List(fields.Str(required=True))
    videos = fields.List(fields.Str(required=True))
    images = fields.List(fields.Str(required=True))
    keywords = fields.List(fields.Str(required=True))
    tags = fields.List(fields.Str(required=True))
    summary = fields.Str(required=True)



import asyncio

from goose3 import Goose

from .http_client import HTTPClient

DEFAULT_USER_AGENT = 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_12_4) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/59.0.3071.115 Safari/537.36'
VALIDATION_LEVELS = ('STRICT', 'DROP', 'ALLOW')


class Article:

    def __init__(self, url: str, user_agent: str = DEFAULT_USER_AGENT, headers: dict = None, proxy: str = None,
                 validation: str = 'STRICT', loop: asyncio.AbstractEventLoop = None):
        self.url = url
        if validation in VALIDATION_LEVELS:
            self.validation = validation
        else:
            raise ValueError('Validation level must be one of STRICT, DROP, or ALLOW')
        if loop:
            self.loop = loop
        else:
            self.loop = asyncio.get_event_loop()
        self.user_agent = user_agent
        self.http_client = HTTPClient(user_agent=user_agent, headers=headers, proxy=proxy)

    async def fetch(self):

        resp = await self.http_client.get(url=self.url, is_text=True)

        article = await self.loop.run_in_executor(None, self._parse_text, resp)

        self.raw_text = article.raw_html
        self.cleaned_text = article.cleaned_text

        article_dict = {
            'url': self.url,
            'title': article.title,
            'authors': article.authors,
            'body': article.cleaned_text,
            'videos': article.movies,
            'images': article.images,
            'tags': article.tags,
            'links': article.links,
            'top_image': article.top_image,
            'tweets': article.tweets,
            'date': article.publish_date
        }

        return article_dict

        # try:
        #     return ArticleSchema(strict=True).load(article_dict).data
        # except ValidationError:
        #     raise SchemaValidationError('Unable to validate response schema')

    def _parse_text(self, text: str):
        g = Goose()
        return g.extract(raw_html=text)

    # async def fetch(self, url: str):
    #     from newspaper import Article, ArticleException
    #
    #     try:
    #         article = Article(url)
    #         article.download()
    #         article.parse()
    #         article.nlp()
    #
    #         article_dict = {
    #             'url': url,
    #             'title': article.title,
    #             'authors': article.authors,
    #             'body': article.text,
    #             'videos': article.movies,
    #             'images': article.images,
    #             'keywords': article.keywords,
    #             'tags': article.tags,
    #             'summary': article.summary
    #         }
    #
    #     except ArticleException:
    #         raise
    #
    #     try:
    #         return ArticleSchema(strict=True).load(article_dict).data
    #     except ValidationError:
    #         raise SchemaValidationError('Unable to validate response schema')



from setuptools import setup

with open('README.md', "r") as fh:
    long_description = fh.read()

setup_args = dict(
    name='aionewsapi',
    version='1.1.1',
    author='Marc Ford',
    url='https://github.com/mfdeux/aionewsapi',
    description='Asyncio client for interacting with newsapi.org',
    long_description=long_description,
    long_description_content_type="text/markdown",
    license='MIT',
    packages=['aionewsapi'],
    classifiers=[
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        "Operating System :: OS Independent",
    ],
    install_requires=[
        'aiohttp',
        'uvloop',
        'marshmallow',
        'python-dateutil'
    ]
)

setup(**setup_args)
